import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import {
  AbstractControl,
  FormArray,
  FormControl,
  FormGroup,
  Validators
} from "@angular/forms";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { UnidadProducto } from "src/app/models/productoempleado";
import { EstadoTarea, Tarea } from "src/app/models/tarea";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";

@Component({
  selector: "app-completar-tarea",
  templateUrl: "./completar-tarea.component.html",
  styleUrls: ["./completar-tarea.component.css"]
})
export class CompletarTareaComponent implements OnInit, OnDestroy {
  public formCompletarTarea: FormGroup;
  public unidades = [];

  public subscriptions = new Subscription();

  constructor(
    @Inject(MAT_DIALOG_DATA) public data,
    private dialogRef: MatDialogRef<CompletarTareaComponent>,
    private snackbarService: SnackbarService,
    private privateAgricultorService: PrivateAgricultorService
  ) {}

  ngOnInit() {
    this.formCompletarTarea = this.inicializarForm();
    this.unidades = Object.keys(UnidadProducto).map(key => ({
      texto: UnidadProducto[key],
      unidad: key
    }));
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }

  public get tarea(): Tarea {
    return this.data.tarea;
  }

  public get fechaMinima() {
    return new Date(this.tarea.fechaInicio);
  }

  private inicializarForm(): FormGroup {
    switch (true) {
      case this.tarea.tipoLabor.necesitaProducto &&
        this.tarea.tipoLabor.recolectaProducto:
        return new FormGroup({
          registroHoras: new FormArray([this.crearFormGroupRegistroHoras()]),
          productoRecogido: new FormGroup({
            nombre: new FormControl("", Validators.required),
            unidad: new FormControl("", Validators.required),
            cantidadRecogida: new FormControl("", [
              Validators.required,
              Validators.min(0.01)
            ])
          }),
          productosEmpleados: new FormArray(
            this.crearFromGroupProductoEmpleado()
          )
        });

      case this.tarea.tipoLabor.necesitaProducto &&
        !this.tarea.tipoLabor.recolectaProducto:
        return new FormGroup({
          registroHoras: new FormArray([this.crearFormGroupRegistroHoras()]),
          productosEmpleados: new FormArray(
            this.crearFromGroupProductoEmpleado()
          )
        });

      case !this.tarea.tipoLabor.necesitaProducto &&
        this.tarea.tipoLabor.recolectaProducto:
        return new FormGroup({
          registroHoras: new FormArray([this.crearFormGroupRegistroHoras()]),
          productoRecogido: new FormGroup({
            nombre: new FormControl("", Validators.required),
            unidad: new FormControl("", Validators.required),
            cantidadRecogida: new FormControl("", [Validators.required])
          })
        });
      case !this.tarea.tipoLabor.necesitaProducto &&
        !this.tarea.tipoLabor.recolectaProducto:
        return new FormGroup({
          registroHoras: new FormArray([this.crearFormGroupRegistroHoras()])
        });
    }
  }

  public get formArrayProductoEmpleados(): FormArray {
    return this.formCompletarTarea.controls["productosEmpleados"] as FormArray;
  }

  public getFormControlProductoEmpleados(index: number): FormGroup {
    return this.formArrayProductoEmpleados.controls[index] as FormGroup;
  }

  public crearFromGroupProductoEmpleado(): AbstractControl[] {
    const array: FormGroup[] = [];
    if (this.tarea.tipoLabor.necesitaProducto) {
      const productosLen: number = (this.tarea as Tarea).productosTareaEmpleados
        .length;
      for (var i = 0; i < productosLen; i++) {
        array.push(
          new FormGroup({
            cantidadEmpleada: new FormControl("", [
              Validators.required,
              Validators.min(0.01)
            ])
          })
        );
      }
    }
    return array;
  }

  public crearFormGroupRegistroHoras(): FormGroup {
    return new FormGroup({
      fecha: new FormControl("", Validators.required),
      horas: new FormControl("", Validators.required)
    });
  }

  public get formArrayRegistrosHoras(): FormArray {
    return this.formCompletarTarea.controls["registroHoras"] as FormArray;
  }

  public anadirRegistroHoras() {
    this.formArrayRegistrosHoras.push(this.crearFormGroupRegistroHoras());
  }

  public eliminarRegistroHoras(index: number) {
    if (this.formArrayRegistrosHoras.controls.length === 1) {
      this.snackbarService.mostrar(
        "Hay que indicar al menos un día.",
        "Ok",
        "snackbar-alert"
      );
    } else {
      this.formArrayRegistrosHoras.removeAt(index);
    }
  }

  public get formGroupProductoRecogido(): FormGroup {
    return this.formCompletarTarea.controls["productoRecogido"] as FormGroup;
  }

  public completarTarea() {
    this.tarea.estadoTarea = EstadoTarea.COMPLETADA;
    this.tarea.registrosHoras = [];
    this.formArrayRegistrosHoras.controls.forEach((fg: FormGroup) => {
      const horas = this.fromHorasMinsToNumber(fg.controls["horas"].value);
      const fecha = fg.controls["fecha"].value;
      this.tarea.registrosHoras.push({ horas: horas, fecha: fecha });
    });

    if (this.tarea.tipoLabor.necesitaProducto) {
      this.formArrayProductoEmpleados.controls.forEach((item, index) => {
        const valor = (item as FormGroup).controls["cantidadEmpleada"].value;
        this.tarea.productosTareaEmpleados[index].cantidadUsadaHa = valor;
      });
    }

    if (this.tarea.tipoLabor.recolectaProducto) {
      const nombre = this.formGroupProductoRecogido.controls["nombre"].value;
      const unidad = this.formGroupProductoRecogido.controls["unidad"].value
        .unidad;
      const cantidadRecogida = this.formGroupProductoRecogido.controls[
        "cantidadRecogida"
      ].value;
      this.tarea.productosTareaRecogidos = [
        {
          nombre: nombre,
          unidadProducto: unidad,
          cantidadRecogidaHa: cantidadRecogida
        }
      ];
    }

    const modificarTarea$ = this.privateAgricultorService.modificarTarea(
      this.tarea
    );
    this.subscriptions.add(
      modificarTarea$.subscribe(
        data => {
          this.data.modificada = true;
          this.data.tarea = data;
          this.dialogRef.close(this.data);
        },
        error => {
          this.tarea.estadoTarea = EstadoTarea.ESTIMADA;
          this.data.modificada = false;
          this.dialogRef.close(this.data);
        }
      )
    );
  }

  private fromHorasMinsToNumber(value: string): number {
    let horas: number = +value.split(":")[0];
    let minutos: number = +value.split(":")[1];
    return horas + minutos / 60;
  }
}
