import { Component, OnDestroy, OnInit } from "@angular/core";
import { FormControl, FormGroup, Validators } from "@angular/forms";
import { forkJoin, of, Subscription } from "rxjs";
import { concatMap, tap } from "rxjs/operators";
import { PrecioProducto } from "src/app/models/precio-producto";
import { ProductoCooperativa } from "src/app/models/producto-cooperativa";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { UsuarioCooperativa } from "src/app/models/usuario-cooperativa";
import { PrivateCooperativaService } from "src/app/services/private-cooperativa.service";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";

@Component({
  selector: "app-cooperativa",
  templateUrl: "./cooperativa.component.html",
  styleUrls: ["./cooperativa.component.css"]
})
export class CooperativaComponent implements OnInit, OnDestroy {
  public cooperativa: UsuarioCooperativa = undefined;
  private subscription: Subscription = new Subscription();
  public formCooperativa;
  public columnas = ["nombre", "precio", "actualizado", "evolucion"];

  constructor(
    private privateAgricultorService: PrivateAgricultorService,
    private privateCooperativaService: PrivateCooperativaService,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    const getCooperativa$ = this.privateAgricultorService.getAgricultor().pipe(
      concatMap(data => {
        if (data.cooperativa !== null) {
          return this.privateCooperativaService.getCooperativa(
            data.cooperativa.identificador
          );
        } else {
          return of(null);
        }
      })
    );

    this.formCooperativa = new FormGroup({
      codigo: new FormControl("", [
        Validators.required,
        Validators.pattern("^@cooperativa[0-9]+@$")
      ])
    });

    this.subscription.add(
      getCooperativa$.subscribe(
        data => {
          this.cooperativa = data;
          this.cooperativa.productosCooperativas.forEach((item, index) => {
            this.cooperativa.productosCooperativas[
              index
            ].preciosProductos = item.preciosProductos.sort(
              this.ordenarPorFechasInverso
            );
          });
        },
        error => {
          this.cooperativa = undefined;
          this.snackbarService.mostrar(
            "No se ha podido recuperar los datos",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }

  public comprobarCodigo() {
    const valor: string = this.formCooperativa.controls["codigo"].value;
    const idCooperativa = valor.replace(/@/g, "").replace("cooperativa", "");
    const modificarUsuario$ = forkJoin(
      this.privateAgricultorService.getAgricultor(),
      this.privateCooperativaService.getCooperativa(+idCooperativa)
    ).pipe(
      tap(data => {
        data[0].cooperativa = data[1];
      }),
      concatMap(data => this.privateAgricultorService.modificarUsuario(data[0]))
    );

    modificarUsuario$.subscribe(
      (data: UsuarioAgricultor) => {
        this.cooperativa = data.cooperativa;
        this.cooperativa.productosCooperativas.forEach(item => {
          item.preciosProductos.sort(this.ordenarPorFechasInverso);
        });
        this.snackbarService.mostrar("Operación completada", "Ok");
      },
      error => {
        this.snackbarService.mostrar(
          "No se pudo realizar la operación, compruebe que el código es el correcto",
          "Ok",
          "snackbar-alert"
        );
      }
    );
  }

  private ordenarPorFechasInverso(
    a: PrecioProducto,
    b: PrecioProducto
  ): number {
    return b.identificador - a.identificador;
  }

  public getIcono(producto: ProductoCooperativa) {
    if (producto.preciosProductos.length === 1) {
      return "remove";
    } else {
      const diferencia =
        producto.preciosProductos[0].precio -
        producto.preciosProductos[1].precio;
      if (diferencia === 0) {
        return "remove";
      } else {
        return diferencia > 0 ? "arrow_upward" : "arrow_downward";
      }
    }
  }
}
