import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import {
  AbstractControl,
  FormArray,
  FormControl,
  FormGroup,
  Validators
} from "@angular/forms";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Apero } from "src/app/models/apero";
import { UnidadProducto } from "src/app/models/productoempleado";
import { EstadoTarea, Tarea } from "src/app/models/tarea";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { aperoAptoTareaValidator } from "src/app/validators/validators";

@Component({
  selector: "app-estimar-tarea",
  templateUrl: "./estimar-tarea.component.html",
  styleUrls: ["./estimar-tarea.component.css"]
})
export class EstimarTareaComponent implements OnInit, OnDestroy {
  constructor(
    @Inject(MAT_DIALOG_DATA) public data,
    private dialogRef: MatDialogRef<EstimarTareaComponent>,
    private privateAgricultorService: PrivateAgricultorService
  ) {}

  private subscriptions: Subscription = new Subscription();

  private aperos: Apero[];
  public formEstimarTarea: FormGroup;
  public unidades: Object[] = [];
  aperosCopia: Apero[];

  ngOnInit() {
    this.formEstimarTarea = new FormGroup({
      fechaInicioEstimada: new FormControl("", Validators.required),
      aperoEmpleado: new FormControl("", Validators.required, [
        aperoAptoTareaValidator(this.tarea)
      ]),
      tiempoEstimado: new FormControl("", Validators.required),
      estimarProductos: new FormArray(this.inicializarFormArray())
    });
    const usuarioLogged: UsuarioAgricultor = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const aperos$ = this.privateAgricultorService.getAperosAgricultor(
      usuarioLogged.identificador
    );
    this.unidades = Object.keys(UnidadProducto).map(key => ({
      texto: UnidadProducto[key],
      unidad: key
    }));
    this.subscriptions.add(
      aperos$.subscribe(
        data => {
          this.aperos = data;
          this.aperosCopia = this.aperos;
        },
        error => {
          this.aperos = [];
        }
      )
    );
    this.formEstimarTarea.controls["aperoEmpleado"].valueChanges.subscribe(
      data => {
        if (typeof data === "string") {
          this.aperosCopia = this.aperos.filter(
            apero =>
              apero.marca.toLowerCase().includes(data.toLowerCase()) ||
              apero.modelo.toLowerCase().includes(data.toLowerCase())
          );
        }
      }
    );
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }

  public get tarea(): Tarea {
    return this.data.tarea;
  }

  public get formArrayProductos(): FormArray {
    return this.formEstimarTarea.controls["estimarProductos"] as FormArray;
  }

  public getFormGroupFormArrayProductos(i: number): FormGroup {
    return this.formArrayProductos.controls[i] as FormGroup;
  }

  public getApero(apero?: Apero) {
    return apero.marca && apero.modelo
      ? `${apero.marca} ${apero.modelo}`
      : undefined;
  }

  public inicializarFormArray(): AbstractControl[] {
    const array: FormGroup[] = [];
    if (this.tarea.tipoLabor.necesitaProducto) {
      const productosLen: number = (this.tarea as Tarea).productosTareaEmpleados
        .length;
      for (var i = 0; i < productosLen; i++) {
        array.push(
          new FormGroup({
            unidad: new FormControl("", Validators.required),
            cantidadEstimada: new FormControl("", [
              Validators.required,
              Validators.min(0.01)
            ]),
            precio: new FormControl("", [
              Validators.required,
              Validators.min(0.01)
            ])
          })
        );
      }
    }
    return array;
  }

  public estimarTarea() {
    this.tarea.apero = this.formEstimarTarea.controls["aperoEmpleado"].value;
    this.tarea.tiempoEstimadoHoras = this.fromHorasMinsToNumber(
      this.formEstimarTarea.controls["tiempoEstimado"].value
    );
    this.tarea.fechaInicio = this.formEstimarTarea.controls[
      "fechaInicioEstimada"
    ].value;
    this.formArrayProductos.controls.forEach((item, index) => {
      const formGroup = this.getFormGroupFormArrayProductos(index);
      this.tarea.productosTareaEmpleados[index].unidadProducto =
        formGroup.controls["unidad"].value.unidad;
      this.tarea.productosTareaEmpleados[index].cantidadEstimadaHa =
        formGroup.controls["cantidadEstimada"].value;
      this.tarea.productosTareaEmpleados[index].precioUnidad =
        formGroup.controls["precio"].value;
    });
    this.tarea.estadoTarea = EstadoTarea.ESTIMADA;
    const modificarTarea$ = this.privateAgricultorService.modificarTarea(
      this.tarea
    );
    this.subscriptions.add(
      modificarTarea$.subscribe(
        data => {
          this.data.modificada = true;
          this.data.tarea = data;
          this.dialogRef.close(this.data);
        },
        error => {
          this.data.modificada = false;
          this.dialogRef.close(this.data);
        }
      )
    );
  }

  private fromHorasMinsToNumber(value: string): number {
    let horas: number = +value.split(":")[0];
    let minutos: number = +value.split(":")[1];
    return horas + minutos / 60;
  }
}
