import { Component, Inject, OnDestroy, OnInit } from "@angular/core";
import { FormArray, FormControl, FormGroup, Validators } from "@angular/forms";
import { MatDialogRef, MAT_DIALOG_DATA } from "@angular/material/dialog";
import { forkJoin, Observable, Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { Parcela } from "src/app/models/parcela";
import { TipoLabor } from "src/app/models/tipolabor";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { PrivatetiposlaborService } from "src/app/services/privatetiposlabor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import {
  agricultorAptoTareaValidator,
  opcionSeleccionadaValidator
} from "src/app/validators/validators";

@Component({
  selector: "app-nuevatarea",
  templateUrl: "./nuevatarea.component.html",
  styleUrls: ["./nuevatarea.component.css"]
})
export class NuevatareaComponent implements OnInit, OnDestroy {
  private subscriptions: Subscription = new Subscription();

  formNuevaTarea: FormGroup;

  necesitaProducto: boolean = false;

  private parcelas: Parcela[] = [];
  tiposLabor: TipoLabor[] = [];
  private agenda: UsuarioAgricultor[] = [];
  parcelasCopia: Parcela[] = [];
  agendaCopia: UsuarioAgricultor[] = [];

  constructor(
    public dialogRef: MatDialogRef<NuevatareaComponent>,
    @Inject(MAT_DIALOG_DATA) public data,
    private privateAgricultorService: PrivateAgricultorService,
    private privateTipoLabor: PrivatetiposlaborService,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    this.formNuevaTarea = new FormGroup(
      {
        parcela: new FormControl(
          "",
          Validators.required,
          opcionSeleccionadaValidator()
        ),
        tipoLabor: new FormControl(""),
        encargado: new FormControl(
          "",
          Validators.required,
          opcionSeleccionadaValidator()
        )
      },
      [],
      agricultorAptoTareaValidator(this.privateAgricultorService)
    );
    const usuarioLogged: UsuarioAgricultor = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
    const llamadaServicios$ = forkJoin(
      this.privateAgricultorService.getParcelasAgricultor(),
      this.privateAgricultorService.getAgendaAgricultor(),
      this.privateTipoLabor.getTiposLabor()
    );
    this.subscriptions.add(
      llamadaServicios$.subscribe(
        data => {
          this.parcelas = data[0];
          this.parcelasCopia = this.parcelas;
          if ((data[1] as Array<UsuarioAgricultor>).length > 0) {
            this.agenda = [usuarioLogged].concat(data[1]);
          } else {
            this.agenda = [usuarioLogged];
          }
          this.tiposLabor = data[2];
          this.agendaCopia = this.agenda;
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido cargar los datos.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
    this.formNuevaTarea.controls["parcela"].valueChanges.subscribe(data => {
      if (typeof data === "string") {
        this.parcelasCopia = this.parcelas.filter(parcela =>
          parcela.nombre.toLowerCase().includes(data.toLowerCase())
        );
      }
    });
    this.formNuevaTarea.controls["encargado"].valueChanges.subscribe(data => {
      if (typeof data === "string") {
        this.agendaCopia = this.agenda.filter(usuario =>
          usuario.nombre.toLowerCase().includes(data.toLowerCase())
        );
      }
    });
  }

  ngOnDestroy(): void {
    this.subscriptions.unsubscribe();
  }

  public get formArrayProductosEmpleados(): FormArray {
    return this.formNuevaTarea.get("productosEmpleados") as FormArray;
  }

  public getFormGroupProductoEmpleado(index: number): FormGroup {
    return this.formArrayProductosEmpleados.controls[index] as FormGroup;
  }

  public nuevoProdutoEmpleado(): FormGroup {
    return new FormGroup({
      nombreProducto: new FormControl("")
    });
  }

  public eliminarProductoEmpleado(index: number) {
    if (
      this.formNuevaTarea.controls["tipoLabor"].value.necesitaProducto &&
      this.formArrayProductosEmpleados.length === 1
    ) {
      this.snackbarService.mostrar(
        "Para este tipo de labor hay que indicar al menos un producto.",
        "Ok",
        "snackbar-alert"
      );
    } else {
      this.formArrayProductosEmpleados.removeAt(index);
    }
  }

  public anadirProducto() {
    this.formArrayProductosEmpleados.push(this.nuevoProdutoEmpleado());
  }

  public actualizarTipoLabor(evento) {
    this.necesitaProducto = evento.value.necesitaProducto;
    if (this.necesitaProducto) {
      this.formNuevaTarea.addControl(
        "productosEmpleados",
        new FormArray([this.nuevoProdutoEmpleado()])
      );
    } else {
      if (this.formArrayProductosEmpleados) {
        this.formNuevaTarea.removeControl("productosEmpleados");
      }
    }
  }

  public getNombre(item?: any) {
    return item.nombre;
  }

  public get nuevaTarea() {
    return this.data.tarea;
  }

  public crearTarea() {
    this.nuevaTarea.parcela = this.formNuevaTarea.controls["parcela"].value;
    this.nuevaTarea.tipoLabor = this.formNuevaTarea.controls["tipoLabor"].value;
    this.nuevaTarea.encargado = this.formNuevaTarea.controls["encargado"].value;
    if (this.necesitaProducto) {
      this.nuevaTarea.productosTareaEmpleados = [];
      this.formArrayProductosEmpleados.controls.forEach(
        (element: FormGroup) => {
          this.nuevaTarea.productosTareaEmpleados.push({
            nombre: element.controls["nombreProducto"].value
          });
        }
      );
    }
    const registrarTarea$ = this.privateAgricultorService.registrarTarea(
      this.nuevaTarea
    );
    this.subscriptions.add(
      registrarTarea$.subscribe(
        data => {
          this.data.creada = true;
          this.data.tarea = data;
          this.dialogRef.close(this.data);
        },
        error => {
          this.data.creada = false;
          this.dialogRef.close(this.data);
        }
      )
    );
  }
}
