import { Component, OnDestroy, OnInit, ViewChild } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import { MatTable, MatTableDataSource } from "@angular/material/table";
import { Subscription } from "rxjs";
import { PrecioProducto } from "src/app/models/precio-producto";
import { ProductoCooperativa } from "src/app/models/producto-cooperativa";
import { PrivateCooperativaService } from "src/app/services/private-cooperativa.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { NuevoProductoComponent } from "../nuevo-producto/nuevo-producto.component";
import { VerDetallesProductoComponent } from "../ver-detalles-producto/ver-detalles-producto.component";

@Component({
  selector: "app-productos-cooperativa",
  templateUrl: "./productos-cooperativa.component.html",
  styleUrls: ["./productos-cooperativa.component.css"]
})
export class ProductosCooperativaComponent implements OnInit, OnDestroy {
  public columnas = [
    "nombre",
    "precioactual",
    "evolucion",
    "eliminar",
    "detalles"
  ];
  public dataSource: MatTableDataSource<ProductoCooperativa> = null;
  @ViewChild(MatTable, { static: true }) productos: MatTable<any>;

  private subscription: Subscription = new Subscription();

  constructor(
    private privateCooperativaService: PrivateCooperativaService,
    private matDialog: MatDialog,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    const getProductos$ = this.privateCooperativaService.getProductosCooperativa();
    this.subscription.add(
      getProductos$.subscribe(
        data => {
          this.dataSource = new MatTableDataSource(data);
        },
        error => {
          this.dataSource = new MatTableDataSource([]);
          this.snackbarService.mostrar(
            "No se ha podido cargar los datos.",
            "OK",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public getPrecioActualProducto(producto: ProductoCooperativa): number {
    return producto.preciosProductos.sort(this.ordenarPorId)[0].precio;
  }

  public getIcono(producto: ProductoCooperativa): string {
    const preciosOrdenados = producto.preciosProductos.sort(this.ordenarPorId);
    if (preciosOrdenados.length === 1) {
      return "remove";
    } else {
      const diferencia =
        preciosOrdenados[0].precio - preciosOrdenados[1].precio;
      if (diferencia === 0) {
        return "remove";
      } else {
        return diferencia > 0 ? "arrow_upward" : "arrow_downward";
      }
    }
  }

  private ordenarPorId(a: PrecioProducto, b: PrecioProducto) {
    return b.identificador - a.identificador;
  }

  public verDetallesProducto(producto: ProductoCooperativa) {
    const dialog = this.matDialog.open(VerDetallesProductoComponent, {
      data: producto,
      height: "95%",
      width: "60%"
    });
    const dialogClosed$ = dialog.afterClosed();
    this.subscription.add(
      dialogClosed$.subscribe(
        data => {
          this.productos.renderRows();
        },
        error => {
          this.productos.renderRows();
        }
      )
    );
  }

  public eliminarProducto(producto: ProductoCooperativa) {
    const eliminarProducto$ = this.privateCooperativaService.eliminarProducto(
      producto
    );
    this.subscription.add(
      eliminarProducto$.subscribe(
        data => {
          const index = this.dataSource.data.indexOf(producto);
          if (index !== -1) {
            this.dataSource.data.splice(index, 1);
            this.productos.renderRows();
            this.snackbarService.mostrar("Producto eliminado", "Ok");
          } else {
            this.snackbarService.mostrar(
              "No se ha podido eliminar el producto",
              "Ok",
              "snackbar-alert"
            );
          }
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha podido eliminar el producto",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public abrirNuevoProducto() {
    const abrirNuevoProducto = this.matDialog.open(NuevoProductoComponent, {
      data: { creado: false, producto: null },
      height: "45%",
      width: "40%"
    });
    const dialogClosed$ = abrirNuevoProducto.afterClosed();
    this.subscription.add(
      dialogClosed$.subscribe(
        data => {
          if (data.creado) {
            this.dataSource.data.push(data.producto);
            this.productos.renderRows();
            this.snackbarService.mostrar("Producto registrado", "Ok");
          } else {
            this.snackbarService.mostrar(
              "No se ha registrado el producto",
              "Ok",
              "snackbar-alert"
            );
          }
        },
        error => {
          this.snackbarService.mostrar(
            "No se ha registrado el producto",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  public filtrar(value: string) {
    this.dataSource.filter = value.toLowerCase();
  }

  ngOnDestroy() {
    this.subscription.unsubscribe();
  }
}
