import { Component, OnDestroy, OnInit } from "@angular/core";
import { FormControl, FormGroup, Validators } from "@angular/forms";
import { MatSnackBar } from "@angular/material/snack-bar";
import { Router } from "@angular/router";
import { Subscription } from "rxjs";
import { UsuarioCooperativa } from "src/app/models/usuario-cooperativa";
import { PublicService } from "src/app/services/public.service";
import {
  repetirPasswordCorrectoValidator,
  userNameUnico
} from "src/app/validators/validators";
import { UsuarioAgricultor } from "../../models/usuario-agricultor";

@Component({
  selector: "app-registro",
  templateUrl: "./registro.component.html",
  styleUrls: ["./registro.component.css"]
})
export class RegistroComponent implements OnInit, OnDestroy {
  formRegistroAgricultor: FormGroup;
  formRegistroCooperativa: FormGroup;
  exito: boolean;

  private subscriptions: Subscription = new Subscription();

  constructor(
    private publicService: PublicService,
    private snackBar: MatSnackBar,
    private router: Router
  ) {}

  ngOnInit() {
    this.formRegistroAgricultor = new FormGroup(
      {
        nombre: new FormControl("", Validators.required),
        username: new FormControl(
          "",
          [Validators.required, Validators.email],
          userNameUnico(this.publicService)
        ),
        password: new FormControl("", Validators.required),
        passwordRepetida: new FormControl("", Validators.required),
        localidad: new FormControl("", Validators.required)
      },
      [],
      repetirPasswordCorrectoValidator()
    );

    this.formRegistroCooperativa = new FormGroup(
      {
        nombre: new FormControl("", Validators.required),
        username: new FormControl(
          "",
          [Validators.required],
          userNameUnico(this.publicService)
        ),
        password: new FormControl("", Validators.required),
        passwordRepetida: new FormControl("", Validators.required),
        nombreResponsable: new FormControl("", Validators.required),
        direccion: new FormControl("", Validators.required),
        numeroTlf: new FormControl("", [Validators.pattern("[0-9]{9}")])
      },
      [],
      repetirPasswordCorrectoValidator()
    );
  }

  ngOnDestroy() {
    this.subscriptions.unsubscribe();
  }

  public registrarAgricultor() {
    const iAgricultor: UsuarioAgricultor = {
      nombre: this.formRegistroAgricultor.controls["nombre"].value,
      username: this.formRegistroAgricultor.controls["username"].value,
      password: this.formRegistroAgricultor.controls["password"].value,
      municipio: this.formRegistroAgricultor.controls["localidad"].value
    };

    let registrarUsuario$ = this.publicService.registrarAgricultor(iAgricultor);
    this.subscriptions.add(
      registrarUsuario$.subscribe(
        data => {
          this.mostrarSnackBar(true);
        },
        error => {
          this.mostrarSnackBar(false);
        }
      )
    );
  }

  public registrarCooperativa() {
    const iCooperativa: UsuarioCooperativa = {
      nombre: this.formRegistroCooperativa.controls["nombre"].value,
      username: this.formRegistroCooperativa.controls["username"].value,
      password: this.formRegistroCooperativa.controls["password"].value,
      nombreResponsable: this.formRegistroCooperativa.controls[
        "nombreResponsable"
      ].value,
      direccionSede: this.formRegistroCooperativa.controls["direccion"].value,
      numeroTlf: this.formRegistroCooperativa.controls["numeroTlf"].value
    };

    let registrarUsuario$ = this.publicService.registrarCooperativa(
      iCooperativa
    );
    this.subscriptions.add(
      registrarUsuario$.subscribe(
        data => {
          this.mostrarSnackBar(true);
        },
        error => {
          this.mostrarSnackBar(false);
        }
      )
    );
  }

  private mostrarSnackBar(exito: boolean) {
    const mensaje: string = exito
      ? "Usuario registrado."
      : "No se ha podido registrar el nuevo usuario.";
    let snackbar = this.snackBar.open(mensaje, "Ok");
    snackbar.afterDismissed().subscribe(() => this.router.navigate(["/login"]));
  }
}
