import { Component, EventEmitter, Input, OnInit, Output } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import pdfMake from "pdfmake/build/pdfmake";
import pdfFonts from "pdfmake/build/vfs_fonts";
import { Constantes } from "src/app/constantes";
import { EstadoTarea, Tarea } from "src/app/models/tarea";
import { UsuarioAgricultor } from "src/app/models/usuario-agricultor";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { DetallesTareaComponent } from "../ver-detalles-tarea/detalles-tarea.component";
import { HttpErrorResponse } from "@angular/common/http";
import { Usuario } from "src/app/models/usuario";
pdfMake.vfs = pdfFonts.pdfMake.vfs;

@Component({
  selector: "app-tarea",
  templateUrl: "./tarea.component.html",
  styleUrls: ["./tarea.component.css"]
})
export class TareaComponent implements OnInit {
  @Input("tarea") tarea: Tarea;
  @Input("bloqueada") bloqueada = false;
  @Output() tareaEliminada: EventEmitter<Tarea> = new EventEmitter<Tarea>();
  public completada: EstadoTarea = EstadoTarea.COMPLETADA;
  public usuarioLogged: Usuario;

  constructor(
    private privateAgricultor: PrivateAgricultorService,
    private snackbarService: SnackbarService,
    private dialog: MatDialog
  ) {}

  ngOnInit() {
    this.usuarioLogged = JSON.parse(
      sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
    );
  }

  eliminarTarea() {
    this.privateAgricultor.eliminarTarea(this.tarea).subscribe(
      data => {
        this.tareaEliminada.emit();
        this.snackbarService.mostrar("Tarea eliminada.", "Ok");
      },
      (error: HttpErrorResponse) => {
        if (error.status === 0) {
          this.snackbarService.mostrar(
            "No se ha podido conectar con el servidor.",
            "Ok",
            "snackbar-alert"
          );
        }
        if (error.status === 400) {
          this.snackbarService.mostrar(error.error, "Ok", "snackbar-alert");
        }
      }
    );
  }

  public isDelegada(): boolean {
    return (
      this.tarea.encargado.identificador !==
      this.tarea.parcela.propietario.identificador
    );
  }

  public verDetalles() {
    let dialog = this.dialog.open(DetallesTareaComponent, {
      data: this.tarea,
      height: "95%",
      width: "60%"
    });
  }

  public puedeGenerarFactura(): boolean {
    return (
      this.tarea.estadoTarea === "COMPLETADA" &&
      this.tarea.encargado.identificador !==
        this.tarea.parcela.propietario.identificador &&
      this.tarea.encargado.identificador === this.usuarioLogged.identificador
    );
  }

  public generarFactura() {
    const content = {
      content: [
        {
          text: "Hoja de costes",
          bold: true,
          fontSize: 20,
          margin: [0, 0, 0, 20]
        },
        {
          columns: [
            {
              margin: [35, 0, 0, 15],
              layout: "noBorders",
              widths: ["auto", "auto"],
              table: {
                body: [
                  [
                    { text: "Código tarea: ", style: "cabecera", noWrap: true },
                    {
                      text: this.tarea.identificador,
                      style: "cabecera",
                      noWrap: true
                    }
                  ],
                  [
                    { text: "Parcela: ", style: "cabecera", noWrap: true },
                    {
                      text: this.tarea.parcela.nombre,
                      style: "cabecera",
                      noWrap: true
                    }
                  ],
                  [
                    {
                      text: "Tamaño parcela:  ",
                      style: "cabecera",
                      noWrap: true
                    },
                    {
                      text: `${this.tarea.parcela.superficieHa} hectáreas`,
                      style: "cabecera",
                      noWrap: true
                    }
                  ],
                  [
                    { text: "Encargado: ", style: "cabecera", noWrap: true },
                    {
                      text: this.tarea.encargado.nombre,
                      style: "cabecera",
                      noWrap: true
                    }
                  ],
                  [
                    {
                      text: "Tipo de labor: ",
                      style: "cabecera",
                      noWrap: true
                    },
                    {
                      text: this.tarea.tipoLabor.nombre,
                      style: "cabecera",
                      noWrap: true
                    }
                  ],
                  [
                    {
                      text: "Fecha de emisión: ",
                      style: "cabecera",
                      noWrap: true
                    },
                    {
                      text: this.prettyPrintFecha(),
                      style: "cabecera",
                      noWrap: true
                    }
                  ]
                ],
                alignment: "center"
              }
            },
            { width: "*", text: "" }
          ]
        },
        {
          text: "Costes",
          fontSize: 15,
          bold: true
        },
        this.getFacturaProductosEmpleados(),
        this.getFacturaAperoUtilizado(),
        this.getTotal()
      ],
      header: {
        text: `Documento generado con ${Constantes.NOMBRE_APP}`,
        alignment: "right"
      },
      info: {
        title: `factura-${this.tarea.parcela.nombre}-${this.tarea.tipoLabor.nombre}`,
        author: `${Constantes.NOMBRE_APP}`
      },
      styles: {
        cabecera: {
          fontSize: 14,
          lineHeight: 1.5
        }
      }
    };
    pdfMake.createPdf(content).open();
  }

  private getFacturaProductosEmpleados() {
    if (this.tarea.tipoLabor.necesitaProducto) {
      const productos = [];
      this.tarea.productosTareaEmpleados.forEach(item => {
        productos.push([
          { text: item.nombre, noWrap: true, alignment: "center" },
          { text: item.cantidadUsadaHa, noWrap: true, alignment: "center" },
          { text: `${item.precioUnidad} €`, noWrap: true, alignment: "center" },
          {
            text: `${item.cantidadUsadaHa *
              this.tarea.parcela.superficieHa *
              item.precioUnidad} €`,
            noWrap: true,
            alignment: "center"
          }
        ]);
      });
      return [
        {
          text: "Productos empleados",
          fontSize: 14,
          margin: [10, 10, 10, 20]
        },
        {
          columns: [
            {
              margin: [35, 0, 0, 10],
              layout: "noBorders",
              widths: ["star-sizing", "auto", "auto"],
              table: {
                body: [
                  [
                    {
                      text: "Nombre del producto ",
                      style: "cabecera",
                      noWrap: true
                    },
                    {
                      text: "Cantidad utilizada ",
                      style: "cabecera",
                      noWrap: true
                    },
                    { text: "Precio unidad ", style: "cabecera", noWrap: true },
                    { text: "Subtotal", style: "cabecera", noWrap: true }
                  ],
                  ...productos
                ]
              }
            }
          ]
        }
      ];
    } else {
      return {};
    }
  }

  private getFacturaAperoUtilizado() {
    return [
      {
        text: "Apero empleado",
        fontSize: 14,
        margin: [10, 15, 10, 20]
      },
      {
        columns: [
          {
            margin: [35, 0, 0, 10],
            layout: "noBorders",
            widths: ["star-sizing", "auto", "auto"],
            table: {
              body: [
                [
                  {
                    text: "Marca del apero",
                    style: "cabecera",
                    noWrap: true
                  },
                  {
                    text: "Modelo del apero",
                    style: "cabecera",
                    noWrap: true
                  },
                  { text: "Precio por ha ", style: "cabecera", noWrap: true },
                  { text: "Subtotal", style: "cabecera", noWrap: true }
                ],
                [
                  {
                    text: this.tarea.apero.marca,
                    noWrap: true,
                    alignment: "center"
                  },
                  {
                    text: this.tarea.apero.modelo,
                    noWrap: true,
                    alignment: "center"
                  },
                  {
                    text: `${this.tarea.apero.costePorHa} €`,
                    noWrap: true,
                    alignment: "center"
                  },
                  {
                    text: `${this.tarea.apero.costePorHa *
                      this.tarea.parcela.superficieHa} €`,
                    noWrap: true,
                    alignment: "center"
                  }
                ]
              ]
            }
          }
        ]
      }
    ];
  }

  private getTotal() {
    const superficie = this.tarea.parcela.superficieHa;
    let totalProductos = 0;
    if (this.tarea.tipoLabor.necesitaProducto) {
      totalProductos = this.tarea.productosTareaEmpleados.reduce(function(
        acum,
        val
      ) {
        return acum + val.cantidadUsadaHa * val.precioUnidad * superficie;
      },
      0);
    }
    const totalApero = this.tarea.apero.costePorHa * superficie;
    return [
      {
        text: "Total",
        fontSize: 14,
        margin: [10, 15, 10, 20]
      },
      {
        columns: [
          {
            margin: [75, 0, 0, 10],
            layout: "noBorders",
            widths: ["star-sizing", "auto", "auto"],
            table: {
              body: [
                [
                  {
                    text: "Coste productos",
                    style: "cabecera",
                    noWrap: true
                  },
                  {
                    text: "Coste apero",
                    style: "cabecera",
                    noWrap: true
                  },
                  {
                    text: "Coste total",
                    style: "cabecera",
                    noWrap: true
                  }
                ],
                [
                  {
                    text: `${totalProductos} €`,
                    noWrap: true,
                    alignment: "center"
                  },
                  {
                    text: `${totalApero} €`,
                    noWrap: true,
                    alignment: "center"
                  },
                  {
                    text: `${totalProductos + totalApero} €`,
                    noWrap: true,
                    alignment: "center"
                  }
                ]
              ]
            }
          }
        ]
      }
    ];
  }

  private prettyPrintFecha(): string {
    const fecha = new Date();
    const dias = [
      "domingo",
      "lunes",
      "martes",
      "miércoles",
      "jueves",
      "viernes",
      "sábado"
    ];
    const meses = [
      "enero",
      "febrero",
      "marzo",
      "abril",
      "mayo",
      "junio",
      "julio",
      "agosto",
      "septiembre",
      "octubre",
      "noviembre",
      "diciembre"
    ];
    const dia = dias[fecha.getDay()];
    const diaMes = fecha.getDate();
    const mes = meses[fecha.getMonth()];
    const ano = fecha.getFullYear();
    return `${dia} ${diaMes} de ${mes} del ${ano}`;
  }
}
