import { CdkDragDrop, transferArrayItem } from "@angular/cdk/drag-drop";
import { Component, OnDestroy, OnInit } from "@angular/core";
import { MatDialog } from "@angular/material/dialog";
import { Subscription } from "rxjs";
import { Constantes } from "src/app/constantes";
import { EstadoTarea, Tarea } from "src/app/models/tarea";
import { Usuario } from "src/app/models/usuario";
import { PrivateAgricultorService } from "src/app/services/privateagricultor.service";
import { SnackbarService } from "src/app/services/snackbaralerts.service";
import { CompletarTareaComponent } from "../completar-tarea/completar-tarea.component";
import { EstimarTareaComponent } from "../estimar-tarea/estimar-tarea.component";
import { NuevatareaComponent } from "../nuevatarea/nuevatarea.component";

@Component({
  selector: "app-tareas",
  templateUrl: "./tareas.component.html",
  styleUrls: ["./tareas.component.css"]
})
export class TareasComponent implements OnInit, OnDestroy {
  tareas$;
  registrarParcela$;
  tareasPteEstimacion: Tarea[];
  tareasEstimadas: Tarea[];
  tareasCompletadas: Tarea[];

  subscription: Subscription = new Subscription();

  private userLogged: Usuario = JSON.parse(
    sessionStorage.getItem(Constantes.USER_LOCAL_STORAGE)
  );

  constructor(
    private privateAgricultor: PrivateAgricultorService,
    private matDialog: MatDialog,
    private snackbarService: SnackbarService
  ) {}

  ngOnInit() {
    this.tareas$ = this.privateAgricultor.getTareasAgricultor();
    this.subscription.add(
      this.tareas$.subscribe(
        (data: Tarea[]) => {
          this.tareasPteEstimacion = data.filter(
            item => item.estadoTarea === EstadoTarea.PTE_ESTIMACION
          );
          this.tareasEstimadas = data.filter(
            item => item.estadoTarea === EstadoTarea.ESTIMADA
          );
          this.tareasCompletadas = data.filter(
            item => item.estadoTarea === EstadoTarea.COMPLETADA
          );
        },
        error => {
          this.tareasPteEstimacion = [];
          this.tareasEstimadas = [];
          this.tareasCompletadas = [];
          this.snackbarService.mostrar(
            "No se ha podido cargar las tareas.",
            "Ok",
            "snackbar-alert"
          );
        }
      )
    );
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  public estimarTarea(event: CdkDragDrop<Tarea[]>) {
    if (event.previousContainer !== event.container) {
      let tareaModificar: Tarea =
        event.previousContainer.data[event.previousIndex];
      const dialog = this.matDialog.open(EstimarTareaComponent, {
        data: { tarea: tareaModificar, modificada: false },
        height: "95%",
        width: "60%"
      });
      const cerrarDialog = dialog.afterClosed();
      this.subscription.add(
        cerrarDialog.subscribe(data => {
          const modificada = data.modificada;
          if (modificada) {
            transferArrayItem(
              event.previousContainer.data,
              event.container.data,
              event.previousIndex,
              event.container.data.length
            );
            this.snackbarService.mostrar("Tarea actualizada", "Ok");
          } else {
            this.snackbarService.mostrar(
              "No se ha podido modificar la tarea",
              "Ok",
              "snackbar-alert"
            );
          }
        })
      );
    }
  }

  public completarTarea(event: CdkDragDrop<Tarea[]>) {
    if (event.previousContainer !== event.container) {
      let tareaModificar: Tarea =
        event.previousContainer.data[event.previousIndex];
      const dialog = this.matDialog.open(CompletarTareaComponent, {
        data: { tarea: tareaModificar, modificada: false },
        height: "95%",
        width: "63%"
      });
      const cerrardialog$ = dialog.afterClosed();
      this.subscription.add(
        cerrardialog$.subscribe(data => {
          const modificada = data.modificada;
          if (modificada) {
            transferArrayItem(
              event.previousContainer.data,
              event.container.data,
              event.previousIndex,
              event.container.data.length
            );
            this.snackbarService.mostrar("Tarea actualizada", "Ok");
          } else {
            this.snackbarService.mostrar(
              "No se ha podido modificar la tarea",
              "Ok",
              "snackbar-alert"
            );
          }
        })
      );
    }
  }

  public abrirFormNuevaTarea() {
    let nuevaTarea: Tarea = {
      encargado: null,
      estadoTarea: EstadoTarea.PTE_ESTIMACION,
      parcela: null,
      tipoLabor: null
    };
    let dialog = this.matDialog.open(NuevatareaComponent, {
      data: { tarea: nuevaTarea, creada: false },
      height: "95%",
      width: "60%"
    });
    const cerrardialog$ = dialog.afterClosed();
    this.subscription.add(
      cerrardialog$.subscribe(data => {
        if (data.creada) {
          this.tareasPteEstimacion.push(data.tarea);
          this.snackbarService.mostrar("Tarea creada", "Ok");
        } else
          this.snackbarService.mostrar(
            "No se ha podido crear la tarea.",
            "Ok",
            "snackbar-alert"
          );
      })
    );
  }

  public eliminarTarea(tarea: Tarea) {
    if (this.tareasPteEstimacion.includes(tarea)) {
      const index = this.tareasPteEstimacion.indexOf(tarea, 0);
      this.tareasPteEstimacion.splice(index, 1);
    } else {
      if (this.tareasEstimadas.includes(tarea)) {
        const index = this.tareasEstimadas.indexOf(tarea, 0);
        this.tareasEstimadas.splice(index, 1);
      }
    }
  }

  public isResponsableTarea(tarea: Tarea): boolean {
    return this.userLogged.identificador === tarea.encargado.identificador;
  }
}
